/*   This file is part of Xsltop.
*
*    Xsltop is free software: you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation.
*
*    Xsltop is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY.  See the
*    GNU General Public License for more details.
*
*    See <http://www.gnu.org/licenses/>.
*/
package xsltop.gui;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.List;
import java.util.NoSuchElementException;

import org.w3c.dom.Node;

import xsltop.Xsltop;
import xsltop.xml.XmlBaseEngine;
import xsltop.xml.XsltopException;
/**
 * Read and keep in memory the config xml file of the xsltop application.
 * <p>Config is a singleton.</p>
 * @author philippe voncken
 *
 */
public class Config {
	private static String FILE_PATH = System.getProperty("user.home")
										+ "/." + Xsltop.NAME + "/config.xml";
	private static String OPEN_FILE_PATH = "/config/openfilepath/text()";
	private static String DIVLOC = "/config/window/dividerlocation/text()";
	private static String WINDOWSIZEX = "/config/window/size/x/text()";
	private static String WINDOWSIZEY = "/config/window/size/y/text()";
	private static String LOGLEVEL = "/config/logger/level/text()";
	private static String NAMESPACEAWARE = "/config/xmloptions/namespaceaware/text()";
	private static String LANGDEFAULT = "/config/dico/default/text()";
	private static String LANG = "/config/dico/lang/text()";
	
	private static String CONFIG_TEMPLATE = "/config.template";
	private XmlBaseEngine doc = null;
	private static Config singleton = null;
	private Config() {
		InputStream file = null;
		try {
			file = new FileInputStream(FILE_PATH);
		} catch (FileNotFoundException exception) {
			file = null;
		}
		if (file == null) {
			file = Config.class.getResourceAsStream(CONFIG_TEMPLATE);
		}
		if (file == null) {
			throw new NoSuchElementException("Configuration no found");
		}
		init(file);
	}
	/**
	 * Initialize the DOM.
	 * @param file config file path
	 */
	private void init(InputStream file) {
		try {
			doc = new XmlBaseEngine(file, false, false);
		} catch (XsltopException exception) {
			doc = null;
			throw new NoSuchElementException("Problem during init configuration");
		}
	}
	/**
	 * getter to the singleton.
	 * @return the Config object
	 */
	public static Config getInstance() {
		if(singleton == null) {
			singleton = new Config();
		}
		return singleton;
	}
	/**
	 * Save the configuration in the XML file.
	 */
	public void saveConfig() {
		try {
			File file = new File(FILE_PATH);
			File parent = file.getParentFile();
			if (!parent.exists()) {
				if(!parent.mkdir()) {
					System.out.println("mkdir impossible: " 
							+ parent.getAbsolutePath());
				}
			}
			doc.exportToFile(file);
		} catch (XsltopException exception) {
			System.out.println(exception.getMessage());
		}
	}
	/**
	 * getter to xml item
	 * @param xpath expression
	 * @param defaultValue if the xpath is bad
	 * @return the search Text 
	 */
	private String getText(String xpath, String defaultValue) {
		String label;
		try {
			Node node = doc.evaluateXPathSingle(xpath);
			if (node != null) {
				label = doc.evaluateXPathSingle(xpath).getNodeValue();
			} else { // on réinitialise la config
				// Ce cas arrive sur une montée de version
				// TODO gérer la sauvegarde des valeurs existantes
				init(Config.class.getResourceAsStream(CONFIG_TEMPLATE));				
				label = getText(xpath, defaultValue);
			}
		} catch (XsltopException exception) {
			label = new String(defaultValue);
			System.out.println(exception.getMessage());
		}
		return label;	
	}
	private List<String> getList(String xpath, String default_lang) {
		List<String> labels;
		try {
			labels = doc.evaluateXPathList(xpath);
		} catch (XsltopException exception) {
			labels = new ArrayList<String>();
			System.out.println(exception.getMessage());
		}
		return labels;
	}
	
	private void setText(String xpath, String value) {
		try {
			doc.setValue(xpath, value);
		} catch (XsltopException exception) {
			init(Config.class.getResourceAsStream(CONFIG_TEMPLATE));
			System.out.println(exception.getMessage());
		}
	}
	/**
	 * getter to the xml item.
	 * @return the search text
	 */
	public String getFilePath() {		
		return getText(OPEN_FILE_PATH, "./");
	}
	/**
	 * setter to the xml item.
	 */
	public void setFilePath(String path) {
		setText(OPEN_FILE_PATH, path);
	}
	/**
	 * getter to the xml item.
	 * @return the search text
	 */
	public List<String> getLang() {		
		return getList(LANG, Xsltop.DEFAULT_LANG);
	}
	/**
	 * getter to the xml item.
	 * @return the search text
	 */
	public String getLangDefault() {		
		return getText(LANGDEFAULT, Xsltop.DEFAULT_LANG);
	}
	/**
	 * getter to the xml item.
	 * @return the search text
	 */
	public String getDividerLocation() {		
		return getText(DIVLOC, "400");
	}
	/**
	 * setter to the xml item.
	 */
	public void setDividerLocation(String dividerLocation) {
		setText(DIVLOC, dividerLocation);
	}
	/**
	 * getter to the xml item.
	 * @return the search text
	 */
	public String getWindowSizeX() {		
		return getText(WINDOWSIZEX, "400");
	}
	/**
	 * setter to the xml item.
	 */
	public void setWindowSizeX(String width) {
		setText(WINDOWSIZEX, width);
	}
	/**
	 * getter to the xml item.
	 * @return the search text
	 */
	public String getWindowSizeY() {		
		return getText(WINDOWSIZEY, "600");
	}
	/**
	 * setter to the xml item.
	 */
	public void setWindowSizeY(String height) {
		setText(WINDOWSIZEY, height);
	}
	/**
	 * getter to the xml item.
	 * @return the search text
	 */
	public String getLogLevel() {		
		return getText(LOGLEVEL, "error");
	}
	/**
	 * setter to the xml item.
	 */
	public void setLogLevel(String level) {
		setText(LOGLEVEL, level);
	}
	/**
	 * setter to the xml item.
	 */
	public void setLangDefault(String lang) {
		setText(LANGDEFAULT, lang);		
	}
	/**
	 * getter to the xml item.
	 * @return the search text
	 */
	public String getNamespaceaware() {		
		return getText(NAMESPACEAWARE, "true");
	}
	/**
	 * setter to the xml item.
	 */
	public void setNamespaceaware(String namespaceaware) {
		setText(NAMESPACEAWARE, namespaceaware);
	}
}
